import { normalizePath } from 'vite';
export function augmentHostWithResources(host, transform, options = {}) {
    const resourceHost = host;
    resourceHost.readResource = function (fileName) {
        const filePath = normalizePath(fileName);
        const content = this.readFile(filePath);
        if (content === undefined) {
            throw new Error('Unable to locate component resource: ' + fileName);
        }
        return content;
    };
    resourceHost.transformResource = async function (data, context) {
        // Only style resources are supported currently
        if (context.type !== 'style') {
            return null;
        }
        if (options.inlineStylesExtension) {
            // Resource file only exists for external stylesheets
            const filename = context.resourceFile ??
                `${context.containingFile.replace(/\.ts$/, `.${options?.inlineStylesExtension}`)}`;
            let stylesheetResult;
            try {
                stylesheetResult = await transform(data, `${filename}?direct`);
            }
            catch (e) {
                console.error(`${e}`);
            }
            return { content: stylesheetResult?.code || '' };
        }
        return null;
    };
}
